<?

 /******************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the Achievements JAS plugin.
 * Copyright 2003-2017 by Matthijs Schuurman
 *
 * The Achievements JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/achievements/
 ******************************************************************************************/


 require_once($_path["include"]."toolkit.php");

 require_once($_path["project"]["plugins"]."achievements/script.php");

 class Plugin_Backend_achievements extends Base_Plugin_Backend
 {
  function __construct(&$plugin)
  {
   global $_path;

   parent::__construct($plugin);

   $this->tk=new Toolkit();

   $this->functionFiles=Array(
    $_path["project"]["plugins"]."achievements/script.php",
    $_path["include"]."script.php",
   );
  }

  function main()
  {
   global $_path;

   $this->backend->addStyleSheet($_path["project"]["style"]."backend.css"); //XXX: relies on FM plugin backend.css

   return true;
  }


  /*
   * Output
   */

  function output()
  {
   $nr=request("achievement");

   //save submitted script
   if(request("s")&&$nr)
   {
    $d=Data::cache("achievements_achievements", "edit", "item", Array("format" => "text"));

    $d->clearValues();
    $d->setValue("nr", $nr);
    $d->setRequestValue("script");
    $d->setRequestValue("disabled");
    $d->submit();

    $this->e->add($d->e);
   }

   //get achievement
   $d=Data::cache("achievements_achievements", "view", "item", Array("format" => "text"));

   if($nr)
   {
    $d->setValue("nr", $nr);

    $d->setShowFields("name", "title", "script", "disabled");

    if(!$d->get())
    {
     $this->e->add($d->e);
     return false;
    }

    $d->next();
   }

   //script editting
   $de=Data::cache("achievements_achievements", "edit", "item", Array("format" => "text"));

   $de->addField("achievement", "nr", "form", "reload", 0, false, false, "Achievement"); //fake achievements list
   $de->addReference("achievement", "achievements_achievements", "nr", "*:0");
   $de->setRequestValue("achievement");

   $this->e->add($de->e);

   if(request("edit")&&request("reload_trigger")!="achievement")
   {
    $de->setRequestValue("script");
    $de->setRequestValue("disabled");
   }
   else if($d->v)
   {
    $de->setValue("script", $d->v["script"]);
    $de->setValue("disabled", $d->v["disabled"]);
   }

   $cms=new CMS(Array("useStyling" => true,
    "anchoring" => false,
    "showSubmitAdditional" => false,
    "showText" => false,
    "showTitle" => false,
    "showTips" => false,
    "useStyling" => false,
   ));
   $cms->setDataForm($de, new Form());

   //render editor
   $r=" <script type=\"text/javascript\">
  var e=new Events();
  e.key('ctrl+enter', testScript);

  function testScript()
  {
   document.getElementsByName(\"test\")[0].click();
  }
</script>\n";

   $r.="<form method=\"post\" enctype=\"multipart/form-data\"><div class=\"script editor invisible\">\n";

   $r.=" <div class=\"half options\">\n";

   //url + options
   $r.="  ".$cms->form->input("hidden", "edit", "1");
   $r.="  ".$cms->form->input("hidden", "dontgetdata", "1");
   $r.="  ".$cms->form->input("hidden", "reload_trigger", "");

   $r.="  ".$cms->field("achievement");
   $r.="  ".$cms->field("owner");
   $r.="  ".$cms->field("disabled");

   //script editor + function reference
   $r.="   ".$cms->field("script");
   $r.=" </div>\n";

   //example output
   $test=(request("test")?$this->testScript($de->getValue("script"), $de->getValue("owner")):"");

   $r.=" <div class=\"half example\">\n";
   $r.="  ".$cms->form->button("test", $this->tk->icon("reload", "small")." ".$this->plugin->l->t("Test"), false, false, "submit");
   $r.="  ".$cms->form->button("s", $this->tk->icon("cache", "small")." ".$this->plugin->l->t("Save"), false, false, "submit");
   $r.="  ".$this->e->show();
   $r.="  ".$test."<br/><br/>";
   $r.="  ".$this->functionReference();
   $r.=" </div>\n";
   $this->e->clear();

   $r.="</div></form>\n";

   return $r;
  }


  function testScript($script, $usernr=false)
  {
   global $_benchmark;

   if($_benchmark) $_benchmark->start("Achievements Script");

   $test=new Achievements_Script();
   $test->set("usernr", $usernr);

   if(!$test->parse("test", $script))
   {
    $this->e->add($test->e);
    if($_benchmark) $_benchmark->stop();
    return false;
   }

   $progress=$test->execute("test");
   $this->e->add($test->e);

   if(!preg_match("/^-?[0-9\.]+$/", $progress)) //nan
   {
    $this->e->add("Script should return a number", "error");
    if($_benchmark) $_benchmark->stop();
    return false;
   }

   $r="<table class=\"test\">";

   $r.="<tr><th>".$this->plugin->l->t("Returns")."</th>";
   $r.="<th>".$this->plugin->l->t("Progress")."</th></tr>";

   $r.="<tr><td class=\"hilite\">".$progress."</td>";
   $r.="<td>".round($progress*100, 1)."%".($progress>=1?" (".$this->plugin->l->t("Complete").")":"")."</td></tr>";

   $r.="</table>";

   if($_benchmark) $_benchmark->stop();
   return $r;
  }


  function functionReference()
  {
   $r="<table class=\"functions\">";

   $cols=6;
   $blacklist=Array("clear", "marker", "convert", "values", "parse", "execute", "debug"
    , "_clean", "setFeed"
   );

   foreach($this->functionFiles as $file)
   {
    if(!($functions=FS::grepS("/^\s+(function|class) ([a-z]\w+)(\(([^\)]+)\))?/smi", $file)))
     continue;

    //render function table
    $l=sizeof($functions[0]);
    for($c=0;$c<$l;$c++)
    {
     if($functions[0][$c]=="class")
     {
      if(strstr($r, "<tr>")) //not the first row
       $r.="<tr><td colspan=\"".$cols."\">&nbsp; </td></tr>";

      $r.="<tr><th class=\"hilite\">".$functions[1][$c]."</th>";
      $r.="<th>Argument 1</th>";
      $r.="<th>Argument 2</th>";
      $r.="<th><i>Optional</i></th>";
      $r.="<th colspan=\"".($cols-4)."\"><i>etc</i></th></tr>";

      if($functions[1][$c]=="Script")
      {
       $r.="<tr><td class=\"hilite\">each</td>";
       $r.="<td>key (array)</td>";
       $r.="<td>key</td>";
       $r.="<td colspan=\"".($cols-3)."\">&nbsp;</td></tr>";

       $r.="<tr><td class=\"hilite\">echo</td>";
       $r.="<td>text</td>";
       $r.="<td colspan=\"".($cols-2)."\">&nbsp;</td></tr>";

       $r.="<tr><td class=\"hilite\">else</td>";
       $r.="<td colspan=\"".($cols-1)."\">&nbsp;</td></tr>";

       $r.="<tr><td class=\"hilite\">exit</td>";
       $r.="<td><i>code<i></td>";
       $r.="<td colspan=\"".($cols-2)."\">&nbsp;</td></tr>";
      }
     }
     else //function
     {
      if(in_array($functions[1][$c], $blacklist)) //skip functions
       continue;

      $colspan=$cols-(substr_count($functions[3][$c], ", ")+2);

      $args=str_replace("$", "", $functions[3][$c]); //remove $
      $args=preg_replace("/([^\s]+)=[^,]+/", "<i>\\1</i>", $args); //mark optional vars
      $args=str_replace(", ", "</td><td>", $args); //make columns

      if($colspan)
       $args.="<td colspan=\"".($colspan)."\">&nbsp; </td>";

      $r.="<tr><td class=\"hilite\">".$functions[1][$c]."</td><td>".$args."</td></tr>";
     }
    }
   }

   $r.="</table>";

   return $r;
  }
 }
?>
